# -*- coding: utf-8 -*-

from .context import pymfx4

import unittest


class AdvancedTestSuite(unittest.TestCase):
    """Advanced test cases."""

    def test_error_binary_unpack(self):
        pass

if __name__ == '__main__':
    unittest.main()
