# -*- coding: utf-8 -*-

from .context import pymfx4

import unittest

class BasicTestSuite(unittest.TestCase):
    """Basic test cases."""

    def test_fullindex(self):
        pass
        
    def test_keyboard(self):
        pass


if __name__ == '__main__':
    unittest.main()