from enum import IntEnum
###########################
# Generated from *Types.h #
###########################

class CAccessControlEnum(IntEnum):
    ACCESS_CONTROL_Level = 0
    ACCESS_CONTROL_Key = 1
    ACCESS_CONTROL_Login = 2
    ACCESS_CONTROL_PasswordService = 3
    ACCESS_CONTROL_PasswordManager = 4
    ACCESS_CONTROL_PasswordOperator = 5
    ACCESS_CONTROL_VALUE_COUNT = 6


class CADConverterEnum(IntEnum):
    ADCONVERTER_CHANNEL0 = 0
    ADCONVERTER_CHANNEL1 = 1
    ADCONVERTER_CHANNEL2 = 2
    ADCONVERTER_CHANNEL3 = 3
    ADCONVERTER_CHANNEL4 = 4
    ADCONVERTER_CHANNEL5 = 5
    ADCONVERTER_CHANNEL6 = 6
    ADCONVERTER_CHANNEL7 = 7
    ADCONVERTER_CHANNEL0mA = 8
    ADCONVERTER_CHANNEL1mA = 9
    ADCONVERTER_CHANNEL2mA = 10
    ADCONVERTER_CHANNEL3mA = 11
    ADCONVERTER_CHANNEL4mA = 12
    ADCONVERTER_CHANNEL5mA = 13
    ADCONVERTER_CHANNEL6mA = 14
    ADCONVERTER_CHANNEL7mA = 15
    ADCONVERTER_SETTINGS_0_BitOrder = 16
    ADCONVERTER_SETTINGS_0_BufferEnable = 17
    ADCONVERTER_SETTINGS_0_RefHi = 18
    ADCONVERTER_SETTINGS_0_IntRef = 19
    ADCONVERTER_SETTINGS_0_ModCLK = 20
    ADCONVERTER_SETTINGS_0_PosInput = 21
    ADCONVERTER_SETTINGS_0_NegInput = 22
    ADCONVERTER_SETTINGS_0_BocsEN = 23
    ADCONVERTER_SETTINGS_0_Idac2Range = 24
    ADCONVERTER_SETTINGS_0_Idac1Range = 25
    ADCONVERTER_SETTINGS_0_Pga = 26
    ADCONVERTER_SETTINGS_0_Idac1Value = 27
    ADCONVERTER_SETTINGS_0_Idac2Value = 28
    ADCONVERTER_SETTINGS_0_OdacValue = 29
    ADCONVERTER_SETTINGS_0_DioValue = 30
    ADCONVERTER_SETTINGS_0_DioDirValue = 31
    ADCONVERTER_SETTINGS_0_DataFormat = 32
    ADCONVERTER_SETTINGS_0_Smode = 33
    ADCONVERTER_SETTINGS_0_DecValue = 34
    ADCONVERTER_SETTINGS_0_OcrValue = 35
    ADCONVERTER_SETTINGS_0_FsrValue = 36
    ADCONVERTER_SETTINGS_0_Mdisp = 37
    ADCONVERTER_SETTINGS_0_MeasureXTimesBeforeIssueingValidMeasurment = 38
    ADCONVERTER_SETTINGS_1_BitOrder = 39
    ADCONVERTER_SETTINGS_1_BufferEnable = 40
    ADCONVERTER_SETTINGS_1_RefHi = 41
    ADCONVERTER_SETTINGS_1_IntRef = 42
    ADCONVERTER_SETTINGS_1_ModCLK = 43
    ADCONVERTER_SETTINGS_1_PosInput = 44
    ADCONVERTER_SETTINGS_1_NegInput = 45
    ADCONVERTER_SETTINGS_1_BocsEN = 46
    ADCONVERTER_SETTINGS_1_Idac2Range = 47
    ADCONVERTER_SETTINGS_1_Idac1Range = 48
    ADCONVERTER_SETTINGS_1_Pga = 49
    ADCONVERTER_SETTINGS_1_Idac1Value = 50
    ADCONVERTER_SETTINGS_1_Idac2Value = 51
    ADCONVERTER_SETTINGS_1_OdacValue = 52
    ADCONVERTER_SETTINGS_1_DioValue = 53
    ADCONVERTER_SETTINGS_1_DioDirValue = 54
    ADCONVERTER_SETTINGS_1_DataFormat = 55
    ADCONVERTER_SETTINGS_1_Smode = 56
    ADCONVERTER_SETTINGS_1_DecValue = 57
    ADCONVERTER_SETTINGS_1_OcrValue = 58
    ADCONVERTER_SETTINGS_1_FsrValue = 59
    ADCONVERTER_SETTINGS_1_Mdisp = 60
    ADCONVERTER_SETTINGS_1_MeasureXTimesBeforeIssueingValidMeasurment = 61
    ADCONVERTER_SETTINGS_2_BitOrder = 62
    ADCONVERTER_SETTINGS_2_BufferEnable = 63
    ADCONVERTER_SETTINGS_2_RefHi = 64
    ADCONVERTER_SETTINGS_2_IntRef = 65
    ADCONVERTER_SETTINGS_2_ModCLK = 66
    ADCONVERTER_SETTINGS_2_PosInput = 67
    ADCONVERTER_SETTINGS_2_NegInput = 68
    ADCONVERTER_SETTINGS_2_BocsEN = 69
    ADCONVERTER_SETTINGS_2_Idac2Range = 70
    ADCONVERTER_SETTINGS_2_Idac1Range = 71
    ADCONVERTER_SETTINGS_2_Pga = 72
    ADCONVERTER_SETTINGS_2_Idac1Value = 73
    ADCONVERTER_SETTINGS_2_Idac2Value = 74
    ADCONVERTER_SETTINGS_2_OdacValue = 75
    ADCONVERTER_SETTINGS_2_DioValue = 76
    ADCONVERTER_SETTINGS_2_DioDirValue = 77
    ADCONVERTER_SETTINGS_2_DataFormat = 78
    ADCONVERTER_SETTINGS_2_Smode = 79
    ADCONVERTER_SETTINGS_2_DecValue = 80
    ADCONVERTER_SETTINGS_2_OcrValue = 81
    ADCONVERTER_SETTINGS_2_FsrValue = 82
    ADCONVERTER_SETTINGS_2_Mdisp = 83
    ADCONVERTER_SETTINGS_2_MeasureXTimesBeforeIssueingValidMeasurment = 84
    ADCONVERTER_SETTINGS_3_BitOrder = 85
    ADCONVERTER_SETTINGS_3_BufferEnable = 86
    ADCONVERTER_SETTINGS_3_RefHi = 87
    ADCONVERTER_SETTINGS_3_IntRef = 88
    ADCONVERTER_SETTINGS_3_ModCLK = 89
    ADCONVERTER_SETTINGS_3_PosInput = 90
    ADCONVERTER_SETTINGS_3_NegInput = 91
    ADCONVERTER_SETTINGS_3_BocsEN = 92
    ADCONVERTER_SETTINGS_3_Idac2Range = 93
    ADCONVERTER_SETTINGS_3_Idac1Range = 94
    ADCONVERTER_SETTINGS_3_Pga = 95
    ADCONVERTER_SETTINGS_3_Idac1Value = 96
    ADCONVERTER_SETTINGS_3_Idac2Value = 97
    ADCONVERTER_SETTINGS_3_OdacValue = 98
    ADCONVERTER_SETTINGS_3_DioValue = 99
    ADCONVERTER_SETTINGS_3_DioDirValue = 100
    ADCONVERTER_SETTINGS_3_DataFormat = 101
    ADCONVERTER_SETTINGS_3_Smode = 102
    ADCONVERTER_SETTINGS_3_DecValue = 103
    ADCONVERTER_SETTINGS_3_OcrValue = 104
    ADCONVERTER_SETTINGS_3_FsrValue = 105
    ADCONVERTER_SETTINGS_3_Mdisp = 106
    ADCONVERTER_SETTINGS_3_MeasureXTimesBeforeIssueingValidMeasurment = 107
    ADCONVERTER_SETTINGS_4_BitOrder = 108
    ADCONVERTER_SETTINGS_4_BufferEnable = 109
    ADCONVERTER_SETTINGS_4_RefHi = 110
    ADCONVERTER_SETTINGS_4_IntRef = 111
    ADCONVERTER_SETTINGS_4_ModCLK = 112
    ADCONVERTER_SETTINGS_4_PosInput = 113
    ADCONVERTER_SETTINGS_4_NegInput = 114
    ADCONVERTER_SETTINGS_4_BocsEN = 115
    ADCONVERTER_SETTINGS_4_Idac2Range = 116
    ADCONVERTER_SETTINGS_4_Idac1Range = 117
    ADCONVERTER_SETTINGS_4_Pga = 118
    ADCONVERTER_SETTINGS_4_Idac1Value = 119
    ADCONVERTER_SETTINGS_4_Idac2Value = 120
    ADCONVERTER_SETTINGS_4_OdacValue = 121
    ADCONVERTER_SETTINGS_4_DioValue = 122
    ADCONVERTER_SETTINGS_4_DioDirValue = 123
    ADCONVERTER_SETTINGS_4_DataFormat = 124
    ADCONVERTER_SETTINGS_4_Smode = 125
    ADCONVERTER_SETTINGS_4_DecValue = 126
    ADCONVERTER_SETTINGS_4_OcrValue = 127
    ADCONVERTER_SETTINGS_4_FsrValue = 128
    ADCONVERTER_SETTINGS_4_Mdisp = 129
    ADCONVERTER_SETTINGS_4_MeasureXTimesBeforeIssueingValidMeasurment = 130
    ADCONVERTER_SETTINGS_5_BitOrder = 131
    ADCONVERTER_SETTINGS_5_BufferEnable = 132
    ADCONVERTER_SETTINGS_5_RefHi = 133
    ADCONVERTER_SETTINGS_5_IntRef = 134
    ADCONVERTER_SETTINGS_5_ModCLK = 135
    ADCONVERTER_SETTINGS_5_PosInput = 136
    ADCONVERTER_SETTINGS_5_NegInput = 137
    ADCONVERTER_SETTINGS_5_BocsEN = 138
    ADCONVERTER_SETTINGS_5_Idac2Range = 139
    ADCONVERTER_SETTINGS_5_Idac1Range = 140
    ADCONVERTER_SETTINGS_5_Pga = 141
    ADCONVERTER_SETTINGS_5_Idac1Value = 142
    ADCONVERTER_SETTINGS_5_Idac2Value = 143
    ADCONVERTER_SETTINGS_5_OdacValue = 144
    ADCONVERTER_SETTINGS_5_DioValue = 145
    ADCONVERTER_SETTINGS_5_DioDirValue = 146
    ADCONVERTER_SETTINGS_5_DataFormat = 147
    ADCONVERTER_SETTINGS_5_Smode = 148
    ADCONVERTER_SETTINGS_5_DecValue = 149
    ADCONVERTER_SETTINGS_5_OcrValue = 150
    ADCONVERTER_SETTINGS_5_FsrValue = 151
    ADCONVERTER_SETTINGS_5_Mdisp = 152
    ADCONVERTER_SETTINGS_5_MeasureXTimesBeforeIssueingValidMeasurment = 153
    ADCONVERTER_SETTINGS_6_BitOrder = 154
    ADCONVERTER_SETTINGS_6_BufferEnable = 155
    ADCONVERTER_SETTINGS_6_RefHi = 156
    ADCONVERTER_SETTINGS_6_IntRef = 157
    ADCONVERTER_SETTINGS_6_ModCLK = 158
    ADCONVERTER_SETTINGS_6_PosInput = 159
    ADCONVERTER_SETTINGS_6_NegInput = 160
    ADCONVERTER_SETTINGS_6_BocsEN = 161
    ADCONVERTER_SETTINGS_6_Idac2Range = 162
    ADCONVERTER_SETTINGS_6_Idac1Range = 163
    ADCONVERTER_SETTINGS_6_Pga = 164
    ADCONVERTER_SETTINGS_6_Idac1Value = 165
    ADCONVERTER_SETTINGS_6_Idac2Value = 166
    ADCONVERTER_SETTINGS_6_OdacValue = 167
    ADCONVERTER_SETTINGS_6_DioValue = 168
    ADCONVERTER_SETTINGS_6_DioDirValue = 169
    ADCONVERTER_SETTINGS_6_DataFormat = 170
    ADCONVERTER_SETTINGS_6_Smode = 171
    ADCONVERTER_SETTINGS_6_DecValue = 172
    ADCONVERTER_SETTINGS_6_OcrValue = 173
    ADCONVERTER_SETTINGS_6_FsrValue = 174
    ADCONVERTER_SETTINGS_6_Mdisp = 175
    ADCONVERTER_SETTINGS_6_MeasureXTimesBeforeIssueingValidMeasurment = 176
    ADCONVERTER_SETTINGS_7_BitOrder = 177
    ADCONVERTER_SETTINGS_7_BufferEnable = 178
    ADCONVERTER_SETTINGS_7_RefHi = 179
    ADCONVERTER_SETTINGS_7_IntRef = 180
    ADCONVERTER_SETTINGS_7_ModCLK = 181
    ADCONVERTER_SETTINGS_7_PosInput = 182
    ADCONVERTER_SETTINGS_7_NegInput = 183
    ADCONVERTER_SETTINGS_7_BocsEN = 184
    ADCONVERTER_SETTINGS_7_Idac2Range = 185
    ADCONVERTER_SETTINGS_7_Idac1Range = 186
    ADCONVERTER_SETTINGS_7_Pga = 187
    ADCONVERTER_SETTINGS_7_Idac1Value = 188
    ADCONVERTER_SETTINGS_7_Idac2Value = 189
    ADCONVERTER_SETTINGS_7_OdacValue = 190
    ADCONVERTER_SETTINGS_7_DioValue = 191
    ADCONVERTER_SETTINGS_7_DioDirValue = 192
    ADCONVERTER_SETTINGS_7_DataFormat = 193
    ADCONVERTER_SETTINGS_7_Smode = 194
    ADCONVERTER_SETTINGS_7_DecValue = 195
    ADCONVERTER_SETTINGS_7_OcrValue = 196
    ADCONVERTER_SETTINGS_7_FsrValue = 197
    ADCONVERTER_SETTINGS_7_Mdisp = 198
    ADCONVERTER_SETTINGS_7_MeasureXTimesBeforeIssueingValidMeasurment = 199
    ADCONVERTER_SETTINGS_SETUP_SaveParameters = 200
    ADCONVERTER_SETTINGS_SETUP_CalibrateOffset = 201
    ADCONVERTER_SETTINGS_SETUP_CalibrateGain = 202
    ADCONVERTER_VALUE_COUNT = 203


class CBoardDigitalInputEnum(IntEnum):
    BOARDDIGITALINPUT_INVERSION = 0
    BOARDDIGITALINPUT_REGISTER = 1
    BOARDDIGITALINPUT_DIGIN1 = 2
    BOARDDIGITALINPUT_DIGIN2 = 3
    BOARDDIGITALINPUT_DIGIN3 = 4
    BOARDDIGITALINPUT_DIGIN4 = 5
    BOARDDIGITALINPUT_DIGIN5 = 6
    BOARDDIGITALINPUT_DIGIN6 = 7
    BOARDDIGITALINPUT_DIGIN7 = 8
    BOARDDIGITALINPUT_DIGIN8 = 9
    BOARDDIGITALINPUT_DIGIN9 = 10
    BOARDDIGITALINPUT_DIGIN10 = 11
    BOARDDIGITALINPUT_DIGIN11 = 12
    BOARDDIGITALINPUT_DIGIN12 = 13
    BOARDDIGITALINPUT_DIGIN13 = 14
    BOARDDIGITALINPUT_DIGIN14 = 15
    BOARDDIGITALINPUT_DIGIN15 = 16
    BOARDDIGITALINPUT_DIGIN16 = 17
    BOARDDIGITALINPUT_DIGIN17 = 18
    BOARDDIGITALINPUT_DIGIN18 = 19
    BOARDDIGITALINPUT_DIGIN19 = 20
    BOARDDIGITALINPUT_DIGIN1DEB = 21
    BOARDDIGITALINPUT_DIGIN2DEB = 22
    BOARDDIGITALINPUT_DIGIN3DEB = 23
    BOARDDIGITALINPUT_DIGIN4DEB = 24
    BOARDDIGITALINPUT_DIGIN5DEB = 25
    BOARDDIGITALINPUT_DIGIN6DEB = 26
    BOARDDIGITALINPUT_DIGIN7DEB = 27
    BOARDDIGITALINPUT_DIGIN8DEB = 28
    BOARDDIGITALINPUT_DIGIN9DEB = 29
    BOARDDIGITALINPUT_DIGIN10DEB = 30
    BOARDDIGITALINPUT_DIGIN11DEB = 31
    BOARDDIGITALINPUT_DIGIN12DEB = 32
    BOARDDIGITALINPUT_DIGIN13DEB = 33
    BOARDDIGITALINPUT_DIGIN14DEB = 34
    BOARDDIGITALINPUT_DIGIN15DEB = 35
    BOARDDIGITALINPUT_DIGIN16DEB = 36
    BOARDDIGITALINPUT_DIGIN17DEB = 37
    BOARDDIGITALINPUT_DIGIN18DEB = 38
    BOARDDIGITALINPUT_DIGIN19DEB = 39
    BOARDDIGITALINPUT_VALUE_COUNT = 40


class CBoardDigitalOutputEnum(IntEnum):
    BOARDDIGITALOUTPUT_DO1 = 0
    BOARDDIGITALOUTPUT_DO2 = 1
    BOARDDIGITALOUTPUT_DO3 = 2
    BOARDDIGITALOUTPUT_DO4 = 3
    BOARDDIGITALOUTPUT_DO5 = 4
    BOARDDIGITALOUTPUT_DO6 = 5
    BOARDDIGITALOUTPUT_DO7 = 6
    BOARDDIGITALOUTPUT_DO8 = 7
    BOARDDIGITALOUTPUT_INVERSION = 8
    BOARDDIGITALOUTPUT_REGISTER = 9
    BOARDDIGITALOUTPUT_VALUE_COUNT = 10


class CBoardDisplayEnum(IntEnum):
    BOARDDISPLAY_LINE1 = 0
    BOARDDISPLAY_LINE2 = 1
    BOARDDISPLAY_CONTRAST_TEMPERATURE = 2
    BOARDDISPLAY_CONTRAST_OFFSET = 3
    BOARDDISPLAY_CONTRAST_FREQUENCY = 4
    BOARDDISPLAY_CONTRAST_DUTY_CYCLE = 5
    BOARDDISPLAY_CONTRAST_NUMBER_OF_COEFFICIENTS = 6
    BOARDDISPLAY_CONTRAST_AD_TEMPERATURE_OFFSET = 7
    BOARDDISPLAY_CONTRAST_AD_TEMPERATURE_MIN = 8
    BOARDDISPLAY_CONTRAST_AD_TEMPERATURE_MAX = 9
    BOARDDISPLAY_CONTRAST_COEFFICIENT_0 = 10
    BOARDDISPLAY_CONTRAST_COEFFICIENT_1 = 11
    BOARDDISPLAY_CONTRAST_COEFFICIENT_2 = 12
    BOARDDISPLAY_CONTRAST_COEFFICIENT_3 = 13
    BOARDDISPLAY_CONTRAST_COEFFICIENT_4 = 14
    BOARDDISPLAY_LINE2_Val1 = 15
    BOARDDISPLAY_LINE2_Val2 = 16
    BOARDDISPLAY_LINE2_Val3 = 17
    BOARDDISPLAY_LINE2_Val4 = 18
    BOARDDISPLAY_LINE2_Val5 = 19
    BOARDDISPLAY_LINE2_Val6 = 20
    BOARDDISPLAY_LINE2_Val7 = 21
    BOARDDISPLAY_LINE2_Val8 = 22
    BOARDDISPLAY_LINE2_Val9 = 23
    BOARDDISPLAY_LINE2_Val10 = 24
    BOARDDISPLAY_LINE2_Val11 = 25
    BOARDDISPLAY_LINE2_Val12 = 26
    BOARDDISPLAY_VALUE_COUNT = 27


class CBoardImpulseEnum(IntEnum):
    BOARDIMPULSE_P1_MODE = 0
    BOARDIMPULSE_P1_PULSES = 1
    BOARDIMPULSE_P1_REVERSE_PULSES = 2
    BOARDIMPULSE_P1_ERROR_PULSES = 3
    BOARDIMPULSE_P1_PULSE_FREQUENCY = 4
    BOARDIMPULSE_P1_PULSE_FREQUENCY_MAX = 5
    BOARDIMPULSE_P2_MODE = 6
    BOARDIMPULSE_P2_PULSES = 7
    BOARDIMPULSE_P2_REVERSE_PULSES = 8
    BOARDIMPULSE_P2_ERROR_PULSES = 9
    BOARDIMPULSE_P2_PULSE_FREQUENCY = 10
    BOARDIMPULSE_P2_PULSE_FREQUENCY_MAX = 11
    BOARDIMPULSE_P3_MODE = 12
    BOARDIMPULSE_P3_PULSES = 13
    BOARDIMPULSE_P3_REVERSE_PULSES = 14
    BOARDIMPULSE_P3_ERROR_PULSES = 15
    BOARDIMPULSE_P3_PULSE_FREQUENCY = 16
    BOARDIMPULSE_P3_PULSE_FREQUENCY_MAX = 17
    BOARDIMPULSE_P4_MODE = 18
    BOARDIMPULSE_P4_PULSES = 19
    BOARDIMPULSE_P4_REVERSE_PULSES = 20
    BOARDIMPULSE_P4_ERROR_PULSES = 21
    BOARDIMPULSE_P4_PULSE_FREQUENCY = 22
    BOARDIMPULSE_P4_PULSE_FREQUENCY_MAX = 23
    BOARDIMPULSE_P5_MODE = 24
    BOARDIMPULSE_P5_PULSES = 25
    BOARDIMPULSE_P5_REVERSE_PULSES = 26
    BOARDIMPULSE_P5_ERROR_PULSES = 27
    BOARDIMPULSE_P5_PULSE_FREQUENCY = 28
    BOARDIMPULSE_P5_PULSE_FREQUENCY_MAX = 29
    BOARDIMPULSE_P6_MODE = 30
    BOARDIMPULSE_P6_PULSES = 31
    BOARDIMPULSE_P6_REVERSE_PULSES = 32
    BOARDIMPULSE_P6_ERROR_PULSES = 33
    BOARDIMPULSE_P6_PULSE_FREQUENCY = 34
    BOARDIMPULSE_P6_PULSE_FREQUENCY_MAX = 35
    BOARDIMPULSE_P7_MODE = 36
    BOARDIMPULSE_P7_PULSES = 37
    BOARDIMPULSE_P7_REVERSE_PULSES = 38
    BOARDIMPULSE_P7_ERROR_PULSES = 39
    BOARDIMPULSE_P7_PULSE_FREQUENCY = 40
    BOARDIMPULSE_P7_PULSE_FREQUENCY_MAX = 41
    BOARDIMPULSE_P8_MODE = 42
    BOARDIMPULSE_P8_PULSES = 43
    BOARDIMPULSE_P8_REVERSE_PULSES = 44
    BOARDIMPULSE_P8_ERROR_PULSES = 45
    BOARDIMPULSE_P8_PULSE_FREQUENCY = 46
    BOARDIMPULSE_P8_PULSE_FREQUENCY_MAX = 47
    BOARDIMPULSE_P1_MAX_COUNT_REVERSE_PULSES = 48
    BOARDIMPULSE_P1_MAX_COUNT_LOST_PULSES = 49
    BOARDIMPULSE_VALUE_COUNT = 50


class CBoardRelayEnum(IntEnum):
    BOARDRELAY_R1 = 0
    BOARDRELAY_R2 = 1
    BOARDRELAY_R3 = 2
    BOARDRELAY_R4 = 3
    BOARDRELAY_R5 = 4
    BOARDRELAY_R6 = 5
    BOARDRELAY_R7 = 6
    BOARDRELAY_R8 = 7
    BOARDRELAY_R9 = 8
    BOARDRELAY_R10 = 9
    BOARDRELAY_R11 = 10
    BOARDRELAY_R12 = 11
    BOARDRELAY_R13 = 12
    BOARDRELAY_R14 = 13
    BOARDRELAY_R15 = 14
    BOARDRELAY_R16 = 15
    BOARDRELAY_INVERSION = 16
    BOARDRELAY_REGISTER = 17
    BOARDRELAY_TEMPERATURE = 18
    BOARDRELAY_TYPE = 19
    BOARDRELAY_ACODE = 20
    BOARDRELAY_VALUE_COUNT = 21


class CClockEnum(IntEnum):
    CLOCK_AliveTime = 0
    CLOCK_DateTime = 1
    CLOCK_DateTimeValue = 2
    CLOCK_VALUE_COUNT = 3


class CCounterEnum(IntEnum):
    COUNTER_PULSES = 0
    COUNTER_PRESSURE = 1
    COUNTER_AVERAGE_PRESSURE = 2
    COUNTER_PRESSURE_P0 = 3
    COUNTER_TEMPERATURE = 4
    COUNTER_TEMPERATURE_AVERAGE = 5
    COUNTER_VOLUME_CORRECTION_TARGET_TEMPERATURE_X = 6
    COUNTER_VT = 7
    COUNTER_VT_PRESSURE_P0 = 8
    COUNTER_SUM_VT = 9
    COUNTER_SUM_VT_PRESSURE_P0 = 10
    COUNTER_VX = 11
    COUNTER_VX_PRESSURE_P0 = 12
    COUNTER_SUM_VX = 13
    COUNTER_SUM_VX_PRESSURE_P0 = 14
    COUNTER_WEIGHT = 15
    COUNTER_SUM_WEIGHT = 16
    COUNTER_DENSITY_T = 17
    COUNTER_DENSITY_X = 18
    COUNTER_DENSITY_T_AVERAGE = 19
    COUNTER_DENSITY_X_AVERAGE = 20
    COUNTER_DENSITY_T_AT_P0_AVERAGE = 21
    COUNTER_DENSITY_X_AT_P0_AVERAGE = 22
    COUNTER_FLOW = 23
    COUNTER_FLOW_DETECTED = 24
    COUNTER_FLOW_DETECTION_MIN_FLOW = 25
    COUNTER_FLOW_DETECTION_MIN_TIME = 26
    COUNTER_VOLUME_CORRECTION_FACTOR_TO_P0 = 27
    COUNTER_VOLUME_CORRECTION_FACTOR_TO_P0_X = 28
    COUNTER_VOLUME_CORRECTION_FACTOR_TO_X = 29
    COUNTER_TEXT = 30
    COUNTER_TABLE = 31
    COUNTER_MODE = 32
    COUNTER_STATE = 33
    COUNTER_FLAGS = 34
    COUNTER_FLAGS_KNOWN = 35
    COUNTER_FLAGS_GOAL = 36
    COUNTER_SUM_PRESSURE = 37
    COUNTER_SUM_DENSITY_T = 38
    COUNTER_SUM_DENSITY_X = 39
    COUNTER_SUM_TEPERATURE = 40
    COUNTER_TEMPERATURE_LAST_VALID = 41
    COUNTER_AMOUNT = 42
    COUNTER_K_FACTOR = 43
    COUNTER_LEAKAGE_PRODUCT = 44
    COUNTER_VALUE_COUNT = 45


class CDAConverterEnum(IntEnum):
    DACONVERTER_OUTPUT0 = 0
    DACONVERTER_OUTPUT1 = 1
    DACONVERTER_OUTPUT2 = 2
    DACONVERTER_OUTPUT3 = 3
    DACONVERTER_OUTPUT4 = 4
    DACONVERTER_OUTPUT5 = 5
    DACONVERTER_OUTPUT6 = 6
    DACONVERTER_OUTPUT7 = 7
    DACONVERTER_OUTPUT0_GAIN0 = 8
    DACONVERTER_OUTPUT1_GAIN0 = 9
    DACONVERTER_OUTPUT2_GAIN0 = 10
    DACONVERTER_OUTPUT3_GAIN0 = 11
    DACONVERTER_OUTPUT4_GAIN0 = 12
    DACONVERTER_OUTPUT5_GAIN0 = 13
    DACONVERTER_OUTPUT6_GAIN0 = 14
    DACONVERTER_OUTPUT7_GAIN0 = 15
    DACONVERTER_OUTPUT0_GAIN1 = 16
    DACONVERTER_OUTPUT1_GAIN1 = 17
    DACONVERTER_OUTPUT2_GAIN1 = 18
    DACONVERTER_OUTPUT3_GAIN1 = 19
    DACONVERTER_OUTPUT4_GAIN1 = 20
    DACONVERTER_OUTPUT5_GAIN1 = 21
    DACONVERTER_OUTPUT6_GAIN1 = 22
    DACONVERTER_OUTPUT7_GAIN1 = 23
    DACONVERTER_OUTPUT0_OFFSET0 = 24
    DACONVERTER_OUTPUT1_OFFSET0 = 25
    DACONVERTER_OUTPUT2_OFFSET0 = 26
    DACONVERTER_OUTPUT3_OFFSET0 = 27
    DACONVERTER_OUTPUT4_OFFSET0 = 28
    DACONVERTER_OUTPUT5_OFFSET0 = 29
    DACONVERTER_OUTPUT6_OFFSET0 = 30
    DACONVERTER_OUTPUT7_OFFSET0 = 31
    DACONVERTER_OUTPUT0_OFFSET1 = 32
    DACONVERTER_OUTPUT1_OFFSET1 = 33
    DACONVERTER_OUTPUT2_OFFSET1 = 34
    DACONVERTER_OUTPUT3_OFFSET1 = 35
    DACONVERTER_OUTPUT4_OFFSET1 = 36
    DACONVERTER_OUTPUT5_OFFSET1 = 37
    DACONVERTER_OUTPUT6_OFFSET1 = 38
    DACONVERTER_OUTPUT7_OFFSET1 = 39
    DACONVERTER_SETUP_COMMAND = 40
    DACONVERTER_VALUE_COUNT = 41


class CDensityEnum(IntEnum):
    DENSITY_FREQUENCY = 0
    DENSITY_PRESSURE = 1
    DENSITY_TEMPERATURE = 2
    DENSITY_KT0 = 3
    DENSITY_KT1 = 4
    DENSITY_K0 = 5
    DENSITY_K1 = 6
    DENSITY_K2 = 7
    DENSITY_K20A = 8
    DENSITY_K20B = 9
    DENSITY_K21A = 10
    DENSITY_K21B = 11
    DENSITY_CALIBRATION_TEMPERATURE = 12
    DENSITY_TEMPERATURE_X = 13
    DENSITY_DENSITY_X = 14
    DENSITY_CALCULATION_RULE = 15
    DENSITY_IS_API = 16
    DENSITY_USED_ASTM_TABLE = 17
    DENSITY_BACKCHANNEL_COUNTER_CONFIG = 18
    DENSITY_DENSITY = 19
    DENSITY_COMPENSATED_TEMPERATURE = 20
    DENSITY_COMPENSATED_TEMPERATURE_PRESSURE = 21
    DENSITY_VALUE_COUNT = 22


class CErrorEnum(IntEnum):
    ERROR_FIRST = 0
    ERROR_LOG_1 = 1
    ERROR_LOG_2 = 2
    ERROR_LOG_3 = 3
    ERROR_LOG_4 = 4
    ERROR_LOG_5 = 5
    ERROR_LOG_6 = 6
    ERROR_LOG_7 = 7
    ERROR_LOG_8 = 8
    ERROR_LOG_9 = 9
    ERROR_LOG_10 = 10
    ERROR_LOG_11 = 11
    ERROR_LOG_12 = 12
    ERROR_LOG_13 = 13
    ERROR_LOG_14 = 14
    ERROR_LOG_15 = 15
    ERROR_LOG_16 = 16
    ERROR_LOG_17 = 17
    ERROR_LOG_18 = 18
    ERROR_LOG_19 = 19
    ERROR_LOG_20 = 20
    ERROR_LOG_21 = 21
    ERROR_LOG_22 = 22
    ERROR_LOG_23 = 23
    ERROR_LOG_24 = 24
    ERROR_LOG_25 = 25
    ERROR_LOG_26 = 26
    ERROR_LOG_27 = 27
    ERROR_LOG_28 = 28
    ERROR_LOG_29 = 29
    ERROR_LOG_30 = 30
    ERROR_LOG_31 = 31
    ERROR_LOG_32 = 32
    ERROR_LOG_33 = 33
    ERROR_LOG_34 = 34
    ERROR_LOG_35 = 35
    ERROR_LOG_36 = 36
    ERROR_LOG_37 = 37
    ERROR_LOG_38 = 38
    ERROR_LOG_39 = 39
    ERROR_LOG_40 = 40
    ERROR_LOG_41 = 41
    ERROR_LOG_42 = 42
    ERROR_LOG_43 = 43
    ERROR_LOG_44 = 44
    ERROR_LOG_45 = 45
    ERROR_LOG_46 = 46
    ERROR_LOG_47 = 47
    ERROR_LOG_48 = 48
    ERROR_LOG_49 = 49
    ERROR_LOG_50 = 50
    ERROR1_NO0 = 51
    ERROR1_NO1 = 52
    ERROR1_NO2 = 53
    ERROR1_NO3 = 54
    ERROR1_NO4 = 55
    ERROR1_NO5 = 56
    ERROR1_NO6 = 57
    ERROR1_NO7 = 58
    ERROR1_NO8 = 59
    ERROR1_NO9 = 60
    ERROR2_NO0 = 61
    ERROR2_NO1 = 62
    ERROR2_NO2 = 63
    ERROR2_NO3 = 64
    ERROR2_NO4 = 65
    ERROR2_NO5 = 66
    ERROR2_NO6 = 67
    ERROR2_NO7 = 68
    ERROR2_NO8 = 69
    ERROR2_NO9 = 70
    ERROR3_NO0 = 71
    ERROR3_NO1 = 72
    ERROR3_NO2 = 73
    ERROR3_NO3 = 74
    ERROR3_NO4 = 75
    ERROR3_NO5 = 76
    ERROR3_NO6 = 77
    ERROR3_NO7 = 78
    ERROR3_NO8 = 79
    ERROR3_NO9 = 80
    ERROR4_NO0 = 81
    ERROR4_NO1 = 82
    ERROR4_NO2 = 83
    ERROR4_NO3 = 84
    ERROR4_NO4 = 85
    ERROR4_NO5 = 86
    ERROR4_NO6 = 87
    ERROR4_NO7 = 88
    ERROR4_NO8 = 89
    ERROR4_NO9 = 90
    ERROR1PROPERTY0 = 91
    ERROR1PROPERTY1 = 92
    ERROR1PROPERTY2 = 93
    ERROR1PROPERTY3 = 94
    ERROR1PROPERTY4 = 95
    ERROR1PROPERTY5 = 96
    ERROR1PROPERTY6 = 97
    ERROR1PROPERTY7 = 98
    ERROR1PROPERTY8 = 99
    ERROR1PROPERTY9 = 100
    ERROR1PROPERTY10 = 101
    ERROR1PROPERTY11 = 102
    ERROR1PROPERTY12 = 103
    ERROR1PROPERTY13 = 104
    ERROR1PROPERTY14 = 105
    ERROR1PROPERTY15 = 106
    ERROR1PROPERTY16 = 107
    ERROR1PROPERTY17 = 108
    ERROR1PROPERTY18 = 109
    ERROR1PROPERTY19 = 110
    ERROR1PROPERTY20 = 111
    ERROR1PROPERTY21 = 112
    ERROR1PROPERTY22 = 113
    ERROR1PROPERTY23 = 114
    ERROR1PROPERTY24 = 115
    ERROR1PROPERTY25 = 116
    ERROR1PROPERTY26 = 117
    ERROR1PROPERTY27 = 118
    ERROR1PROPERTY28 = 119
    ERROR1PROPERTY29 = 120
    ERROR2PROPERTY0 = 121
    ERROR2PROPERTY1 = 122
    ERROR2PROPERTY2 = 123
    ERROR2PROPERTY3 = 124
    ERROR2PROPERTY4 = 125
    ERROR2PROPERTY5 = 126
    ERROR2PROPERTY6 = 127
    ERROR2PROPERTY7 = 128
    ERROR2PROPERTY8 = 129
    ERROR2PROPERTY9 = 130
    ERROR2PROPERTY10 = 131
    ERROR2PROPERTY11 = 132
    ERROR2PROPERTY12 = 133
    ERROR2PROPERTY13 = 134
    ERROR2PROPERTY14 = 135
    ERROR2PROPERTY15 = 136
    ERROR2PROPERTY16 = 137
    ERROR2PROPERTY17 = 138
    ERROR2PROPERTY18 = 139
    ERROR2PROPERTY19 = 140
    ERROR2PROPERTY20 = 141
    ERROR2PROPERTY21 = 142
    ERROR2PROPERTY22 = 143
    ERROR2PROPERTY23 = 144
    ERROR2PROPERTY24 = 145
    ERROR2PROPERTY25 = 146
    ERROR2PROPERTY26 = 147
    ERROR2PROPERTY27 = 148
    ERROR2PROPERTY28 = 149
    ERROR2PROPERTY29 = 150
    ERROR3PROPERTY0 = 151
    ERROR3PROPERTY1 = 152
    ERROR3PROPERTY2 = 153
    ERROR3PROPERTY3 = 154
    ERROR3PROPERTY4 = 155
    ERROR3PROPERTY5 = 156
    ERROR3PROPERTY6 = 157
    ERROR3PROPERTY7 = 158
    ERROR3PROPERTY8 = 159
    ERROR3PROPERTY9 = 160
    ERROR3PROPERTY10 = 161
    ERROR3PROPERTY11 = 162
    ERROR3PROPERTY12 = 163
    ERROR3PROPERTY13 = 164
    ERROR3PROPERTY14 = 165
    ERROR3PROPERTY15 = 166
    ERROR3PROPERTY16 = 167
    ERROR3PROPERTY17 = 168
    ERROR3PROPERTY18 = 169
    ERROR3PROPERTY19 = 170
    ERROR3PROPERTY20 = 171
    ERROR3PROPERTY21 = 172
    ERROR3PROPERTY22 = 173
    ERROR3PROPERTY23 = 174
    ERROR3PROPERTY24 = 175
    ERROR3PROPERTY25 = 176
    ERROR3PROPERTY26 = 177
    ERROR3PROPERTY27 = 178
    ERROR3PROPERTY28 = 179
    ERROR3PROPERTY29 = 180
    ERROR4PROPERTY0 = 181
    ERROR4PROPERTY1 = 182
    ERROR4PROPERTY2 = 183
    ERROR4PROPERTY3 = 184
    ERROR4PROPERTY4 = 185
    ERROR4PROPERTY5 = 186
    ERROR4PROPERTY6 = 187
    ERROR4PROPERTY7 = 188
    ERROR4PROPERTY8 = 189
    ERROR4PROPERTY9 = 190
    ERROR4PROPERTY10 = 191
    ERROR4PROPERTY11 = 192
    ERROR4PROPERTY12 = 193
    ERROR4PROPERTY13 = 194
    ERROR4PROPERTY14 = 195
    ERROR4PROPERTY15 = 196
    ERROR4PROPERTY16 = 197
    ERROR4PROPERTY17 = 198
    ERROR4PROPERTY18 = 199
    ERROR4PROPERTY19 = 200
    ERROR4PROPERTY20 = 201
    ERROR4PROPERTY21 = 202
    ERROR4PROPERTY22 = 203
    ERROR4PROPERTY23 = 204
    ERROR4PROPERTY24 = 205
    ERROR4PROPERTY25 = 206
    ERROR4PROPERTY26 = 207
    ERROR4PROPERTY27 = 208
    ERROR4PROPERTY28 = 209
    ERROR4PROPERTY29 = 210
    ERROR_VALUE_COUNT = 211


class CInternalADCEnum(IntEnum):
    INTERNAL_ADC_DIE_TEMPERATURE = 0
    INTERNAL_ADC_EX_DISPLAY_TEMPERATURE = 1
    INTERNAL_ADC_VALUE_COUNT = 2


class CLogEnum(IntEnum):
    LOG_FLAGS = 0
    LOG_ITEMS_MAX = 1
    LOG_ITEMS_CURRENT = 2
    LOG_MASK = 3
    LOG_SELECT = 4
    LOG_MESSAGE = 5
    LOG_DEBUG = 6
    LOG_VALUE_COUNT = 7


class CMMemoryEnum(IntEnum):
    MEMORY_SECTION = 0
    MEMORY_POSITION = 1
    MEMORY_LENGTH = 2
    MEMORY_OPERATION = 3
    MEMORY_ERROR = 4
    MEMORY_RX = 5
    MEMORY_TX = 6
    MEMORY_VALUE_COUNT = 7


class CPIDControlEnum(IntEnum):
    PIDCONTROL_PFACTOR = 0
    PIDCONTROL_IFACTOR = 1
    PIDCONTROL_DFACTOR = 2
    PIDCONTROL_PVALUE = 3
    PIDCONTROL_IVALUE = 4
    PIDCONTROL_DVALUE = 5
    PIDCONTROL_INPUT = 6
    PIDCONTROL_OUTPUT = 7
    #PIDCONTROL_ENABLE = 8
    PIDCONTROL_VALUE_COUNT = 9


class CPressureEnum(IntEnum):
    PRESSURE_MODE = 0
    PRESSURE_OUT_CALCULATION_LOW_BOUNDARY = 1
    PRESSURE_OUT_CALCULATION_HIGH_BOUNDARY = 2
    PRESSURE_IN_CALCULATION_LOW_BOUNDARY = 3
    PRESSURE_IN_CALCULATION_HIGH_BOUNDARY = 4
    PRESSURE_ERROR_MIN = 5
    PRESSURE_ERROR_MAX = 6
    PRESSURE_ERROR_SHIFT = 7
    PRESSURE_IN = 8
    PRESSURE_OUT = 9
    PRESSURE_VALUE_COUNT = 10


class CProcessControlEnum(IntEnum):
    PROCESSCONTROL_MODE = 0
    PROCESSCONTROL_STATE = 1
    PROCESSCONTROL_COMMAND = 2
    PROCESSCONTROL_TRANSACTION = 3
    PROCESSCONTROL_BATCH = 4
    PROCESSCONTROL_RAMP_STATE = 5
    PROCESSCONTROL_RAMP_COMMAND = 6
    PROCESSCONTROL_RAMP_LF = 7
    PROCESSCONTROL_RAMP_LFD = 8
    PROCESSCONTROL_RAMP_LF_QUANTITY = 9
    PROCESSCONTROL_RAMP_HF = 10
    PROCESSCONTROL_RAMP_HFD = 11
    PROCESSCONTROL_RAMP_EF1 = 12
    PROCESSCONTROL_RAMP_EF1D = 13
    PROCESSCONTROL_RAMP_EF1_QUANTITY = 14
    PROCESSCONTROL_RAMP_EF2 = 15
    PROCESSCONTROL_RAMP_EF2D = 16
    PROCESSCONTROL_RAMP_EF2_QUANTITY = 17
    PROCESSCONTROL_RAMP_OUTPUT = 18
    PROCESSCONTROL_PID_STATE = 19
    PROCESSCONTROL_PID_COMMAND = 20
    PROCESSCONTROL_PID_P = 21
    PROCESSCONTROL_PID_I = 22
    PROCESSCONTROL_PID_D = 23
    PROCESSCONTROL_PID_P_VALUE = 24
    PROCESSCONTROL_PID_I_VALUE = 25
    PROCESSCONTROL_PID_D_VALUE = 26
    PROCESSCONTROL_PID_OUTPUT = 27
    PROCESSCONTROL_ANALOG_OUT = 28
    PROCESSCONTROL_FLOW = 29
    PROCESSCONTROL_VT = 30
    PROCESSCONTROL_VX = 31
    PROCESSCONTROL_WG = 32
    PROCESSCONTROL_BATCH_MINIMAL = 33
    PROCESSCONTROL_BATCH_REMAINING = 34
    PROCESSCONTROL_TRANSACTION_REMAINING = 35
    PROCESSCONTROL_SUM_VT = 36
    PROCESSCONTROL_SUM_VX = 37
    PROCESSCONTROL_SUM_WG = 38
    PROCESSCONTROL_PRESET_TYPE = 39
    PROCESSCONTROL_PRODUCT_ARRAY_LNK = 40
    PROCESSCONTROL_PRODUCT_SELECTED = 41
    PROCESSCONTROL_PRESSURE = 42
    PROCESSCONTROL_AVERAGE_PRESSURE = 43
    PROCESSCONTROL_TEMPERATURE = 44
    PROCESSCONTROL_TEMPERATURE_AVERAGE = 45
    PROCESSCONTROL_VT_PRESSURE_P0 = 46
    PROCESSCONTROL_SUM_VT_PRESSURE_P0 = 47
    PROCESSCONTROL_VX_PRESSURE_P0 = 48
    PROCESSCONTROL_SUM_VX_PRESSURE_P0 = 49
    PROCESSCONTROL_DENSITY_T = 50
    PROCESSCONTROL_DENSITY_X = 51
    PROCESSCONTROL_DENSITY_T_AVERAGE = 52
    PROCESSCONTROL_DENSITY_X_AVERAGE = 53
    PROCESSCONTROL_DENSITY_T_AT_P0_AVERAGE = 54
    PROCESSCONTROL_DENSITY_X_AT_P0_AVERAGE = 55
    PROCESSCONTROL_FLOW_DETECTED = 56
    PROCESSCONTROL_VOLUME_CORRECTION_FACTOR_TO_P0 = 57
    PROCESSCONTROL_VOLUME_CORRECTION_FACTOR_TO_P0_X = 58
    PROCESSCONTROL_VOLUME_CORRECTION_FACTOR_TO_X = 59
    PROCESSCONTROL_VT_ERRORLESS = 60
    PROCESSCONTROL_VX_ERRORLESS = 61
    PROCESSCONTROL_WG_ERRORLESS = 62
    PROCESSCONTROL_TEMPERATURE_AVERAGE_ERRORLESS = 63
    PROCESSCONTROL_DENSITY_T_AVERAGE_ERRORLESS = 64
    PROCESSCONTROL_DENSITY_X_AVERAGE_ERRORLESS = 65
    PROCESSCONTROL_VOUCHERTEXT1 = 66
    PROCESSCONTROL_VOUCHERTEXT2 = 67
    PROCESSCONTROL_HOSTTEXT1 = 68
    PROCESSCONTROL_HOSTTEXT2 = 69
    PROCESSCONTROL_HOSTTEXT3 = 70
    PROCESSCONTROL_TIMESTART = 71
    PROCESSCONTROL_TIMEEND = 72
    PROCESSCONTROL_CHECKSUM1 = 73
    PROCESSCONTROL_CHECKSUM2 = 74
    PROCESSCONTROL_CHECKSUM3 = 75
    PROCESSCONTROL_ERROR_FLAG = 76
    PROCESSCONTROL_FLAGS = 77
    PROCESSCONTROL_NUMBER_OF_ENTRIES = 78
    PROCESSCONTROL_ORIGINAL = 79
    PROCESSCONTROL_VOUCHER_NUMBER = 80
    PROCESSCONTROL_RESET_CODE = 81
    PROCESSCONTROL_ERROR_CODE = 82
    PROCESSCONTROL_WORKING_MODE = 83
    PROCESSCONTROL_ADDITIVE_VOLUME = 84
    PROCESSCONTROL_ADDITIVE_RATE = 85
    PROCESSCONTROL_ADDITIVE_TARGET_RATE = 86
    PROCESSCONTROL_ADDITIVE_NUMBER = 87
    PROCESSCONTROL_ADDITIVE_SUM = 88
    PROCESSCONTROL_PRODUCT_TEXT = 89
    PROCESSCONTROL_NODE_ID = 90
    PROCESSCONTROL_ERROR_NUMBERS = 91
    PROCESSCONTROL_VOLUME_CORRECTION_FACTOR = 92
    PROCESSCONTROL_USER_FLOAT_1 = 93
    PROCESSCONTROL_USER_FLOAT_2 = 94
    PROCESSCONTROL_USER_FLOAT_3 = 95
    PROCESSCONTROL_USER_FLOAT_4 = 96
    PROCESSCONTROL_USER_FLOAT_5 = 97
    PROCESSCONTROL_USER_FLOAT_6 = 98
    PROCESSCONTROL_USER_UINT_1 = 99
    PROCESSCONTROL_USER_UINT_2 = 100
    PROCESSCONTROL_USER_UINT_3 = 101
    PROCESSCONTROL_USER_UINT_4 = 102
    PROCESSCONTROL_RAW_PULSES = 103
    PROCESSCONTROL_VOLUME_CORRECTION_FACTOR_TODO = 104
    PROCESSCONTROL_K_FACTOR = 105
    PROCESSCONTROL_METER_FACTOR = 106
    PROCESSCONTROL_VOUCHER_COPY = 107
    PROCESSCONTROL_DISPLAYTEXT = 108
    PROCESSCONTROL_VALUE_COUNT = 109


class CProductEnum(IntEnum):
    PRODUCT_PRESSURE = 0
    PRODUCT_AVERAGE_PRESSURE = 1
    PRODUCT_TEMPERATURE = 2
    PRODUCT_TEMPERATURE_AVERAGE = 3
    PRODUCT_VT_IN = 4
    PRODUCT_VT_PRESSURE_P0_IN = 5
    PRODUCT_SUM_VT_IN = 6
    PRODUCT_SUM_VT_PRESSURE_P0_IN = 7
    PRODUCT_VX_IN = 8
    PRODUCT_VX_PRESSURE_P0_IN = 9
    PRODUCT_SUM_VX_IN = 10
    PRODUCT_SUM_VX_PRESSURE_P0_IN = 11
    PRODUCT_WEIGHT_IN = 12
    PRODUCT_SUM_WEIGHT_IN = 13
    PRODUCT_VT_SNAPSHOT = 14
    PRODUCT_VT_PRESSURE_P0_SNAPSHOT = 15
    PRODUCT_SUM_VT_SNAPSHOT = 16
    PRODUCT_SUM_VT_PRESSURE_P0_SNAPSHOT = 17
    PRODUCT_VX_SNAPSHOT = 18
    PRODUCT_VX_PRESSURE_P0_SNAPSHOT = 19
    PRODUCT_SUM_VX_SNAPSHOT = 20
    PRODUCT_SUM_VX_PRESSURE_P0_SNAPSHOT = 21
    PRODUCT_WEIGHT_SNAPSHOT = 22
    PRODUCT_SUM_WEIGHT_SNAPSHOT = 23
    PRODUCT_VT_OUT = 24
    PRODUCT_VT_PRESSURE_P0_OUT = 25
    PRODUCT_SUM_VT_OUT = 26
    PRODUCT_SUM_VT_PRESSURE_P0_OUT = 27
    PRODUCT_VX_OUT = 28
    PRODUCT_VX_PRESSURE_P0_OUT = 29
    PRODUCT_SUM_VX_OUT = 30
    PRODUCT_SUM_VX_PRESSURE_P0_OUT = 31
    PRODUCT_WEIGHT_OUT = 32
    PRODUCT_SUM_WEIGHT_OUT = 33
    PRODUCT_DENSITY_T = 34
    PRODUCT_DENSITY_X = 35
    PRODUCT_DENSITY_T_AVERAGE = 36
    PRODUCT_DENSITY_X_AVERAGE = 37
    PRODUCT_DENSITY_T_AT_P0_AVERAGE = 38
    PRODUCT_DENSITY_X_AT_P0_AVERAGE = 39
    PRODUCT_FLOW = 40
    PRODUCT_FLOW_DETECTED = 41
    PRODUCT_VOLUME_CORRECTION_FACTOR_TO_P0 = 42
    PRODUCT_VOLUME_CORRECTION_FACTOR_TO_P0_X = 43
    PRODUCT_VOLUME_CORRECTION_FACTOR_TO_X = 44
    PRODUCT_VT_ERRORLESS = 45
    PRODUCT_VX_ERRORLESS = 46
    PRODUCT_WG_ERRORLESS = 47
    PRODUCT_TEMPERATURE_AVERAGE_ERRORLESS = 48
    PRODUCT_DENSITY_T_AVERAGE_ERRORLESS = 49
    PRODUCT_DENSITY_X_AVERAGE_ERRORLESS = 50
    PRODUCT_TEXT = 51
    PRODUCT_TABLE = 52
    PRODUCT_MODE = 53
    PRODUCT_STATE = 54
    PRODUCT_ERROR_FLAGS = 55
    PRODUCT_COUNTER_SELECTION_LNK = 56
    PRODUCT_VOLUME_WITHOUT_ORDER_MAX = 57
    PRODUCT_K_FACTOR = 58
    PRODUCT_START_TIME = 59
    PRODUCT_END_TIME = 60
    PRODUCT_NAME = 61
    PRODUCT_VALUE_COUNT = 62


class CProtocolMFX4Enum(IntEnum):
    PROTOCOLMFX4_TX = 0
    PROTOCOLMFX4_RX = 1
    PROTOCOLMFX4_SEND_REQUEST = 2
    PROTOCOLMFX4_RECEIVE_REQUEST = 3
    PROTOCOLMFX4_VALUE_COUNT = 4


class CPulseGeneratorEnum(IntEnum):
    PULSEGENERATOR_MAXIMUM_FREQUENCY = 0
    PULSEGENERATOR_OUTPUT_TARGET = 1
    PULSEGENERATOR_OUTPUT_ACTUAL = 2
    PULSEGENERATOR_OUTPUT_FREQUENCY = 3
    PULSEGENERATOR_AMOUNT = 4
    PULSEGENERATOR_KFACTOR = 5
    PULSEGENERATOR_VALUE_COUNT = 6


class CPulseEnum(IntEnum):
    PULSE_FREQUENCY1 = 0
    PULSE_KFACTOR1 = 1
    PULSE_FREQUENCY2 = 2
    PULSE_KFACTOR2 = 3
    PULSE_FREQUENCY3 = 4
    PULSE_KFACTOR3 = 5
    PULSE_FREQUENCY4 = 6
    PULSE_KFACTOR4 = 7
    PULSE_FREQUENCY5 = 8
    PULSE_KFACTOR5 = 9
    PULSE_FREQUENCY6 = 10
    PULSE_KFACTOR6 = 11
    PULSE_TYPE = 12
    PULSE_FREQUENCY_MIN = 13
    PULSE_FREQUENCY_MAX = 14
    PULSE_DIFFERENCE = 15
    PULSE_ABSOLUTE = 16
    PULSE_AVERAGE = 17
    PULSE_OUTPUT = 18
    PULSE_VALUE_COUNT = 19


class CSerialEnum(IntEnum):
    SERIAL_FLAGS = 0
    SERIAL_OPEN = 1
    SERIAL_BAUDRATE = 2
    SERIAL_DATABITS = 3
    SERIAL_PARITY = 4
    SERIAL_STOP_BITS = 5
    SERIAL_TX = 6
    SERIAL_RX = 7
    SERIAL_VALUE_COUNT = 8


class CSystemEnum(IntEnum):
    SYSTEM_SW_VERSION = 0
    SYSTEM_SW_DATE = 1
    SYSTEM_SW_TIME = 2
    SYSTEM_SW_WM_Version = 3
    SYSTEM_SW_BOOTROM_VERSION = 4
    SYSTEM_SW_CALIBRATION_CRC = 5
    SYSTEM_CPU_CLOCK = 6
    SYSTEM_CPU_RAMSIZE = 7
    SYSTEM_CPUAPP_UPLOADSIZE = 8
    SYSTEM_CPUAPP_UPLOADCRC = 9
    SYSTEM_CPUAPP_SETMODE = 10
    SYSTEM_CPUAPP_NODEID = 11
    SYSTEM_CPUAPP_CANBUS_BAUDRATE = 12
    SYSTEM_SAVE_ALL_PARAMETERS = 13
    SYSTEM_SAVE_APPLICATION_PARAMETERS = 14
    SYSTEM_SAVE_COMMUNICATION_PARAMETERS = 15
    SYSTEM_RESTORE_DEFAULT_PARAMETERS = 16
    SYSTEM_INTERNAL_DIE_TEMPERATURE = 17
    SYSTEM_AIRLIFT = 18
    SYSTEM_UNIT_TEMPERATURE = 19
    SYSTEM_UNIT_DENSITY = 20
    SYSTEM_UNIT_WEIGHT = 21
    SYSTEM_UNIT_VOLUME = 22
    SYSTEM_UNIT_PRESSURE = 23
    SYSTEM_MANUFACTURER_DEVICE_NAME = 24
    SYSTEM_HARDWARE_BOARD_VERSION = 25
    SYSTEM_VALUE_COUNT = 26


class CTemperatureEnum(IntEnum):
    TEMPERATURE_ANALOG_IN_0 = 0
    TEMPERATURE_ANALOG_IN_1 = 1
    TEMPERATURE_RESISTANCE = 2
    TEMPERATURE_TEMPERATURE = 3
    TEMPERATURE_MIN = 4
    TEMPERATURE_MAX = 5
    TEMPERATURE_SHIFT = 6
    TEMPERATURE_MODE = 7
    TEMPERATURE_VALUE_COUNT = 8


class CVoucherEnum(IntEnum):
    VOUCHER_PROCESSCONTROL = 0
    VOUCHER_NOT_PRINTED = 1
    VOUCHER_ORIGINAL = 2
    VOUCHER_NUMBER = 3
    VOUCHER_RESET_CODE = 4
    VOUCHER_ERROR_CODE = 5
    VOUCHER_VT = 6
    VOUCHER_VX = 7
    VOUCHER_WG = 8
    VOUCHER_SUM_VT = 9
    VOUCHER_SUM_VX = 10
    VOUCHER_SUM_WG = 11
    VOUCHER_WORKING_MODE = 12
    VOUCHER_TEMPERATURE_AVERAGE = 13
    VOUCHER_DENSITY_T_AVERAGE = 14
    VOUCHER_DENSITY_X_AVERAGE = 15
    VOUCHER_ADDITIVE_VOLUME = 16
    VOUCHER_ADDITIVE_RATE = 17
    VOUCHER_ADDITIVE_TARGET_RATE = 18
    VOUCHER_ADDITIVE_NUMBER = 19
    VOUCHER_ADDITIVE_SUM = 20
    VOUCHER_ERROR_FLAG = 21
    VOUCHER_CHECKSUM_CALIBRATION = 22
    VOUCHER_TIMEEND = 23
    VOUCHER_VOUCHERTEXT2 = 24
    VOUCHER_VT_ERRORLESS = 25
    VOUCHER_VX_ERRORLESS = 26
    VOUCHER_WG_ERRORLESS = 27
    VOUCHER_TEMPERATURE_AVERAGE_ERRORLESS = 28
    VOUCHER_DENSITY_T_AVERAGE_ERRORLESS = 29
    VOUCHER_DENSITY_X_AVERAGE_ERRORLESS = 30
    VOUCHER_TEMPERATURE = 31
    VOUCHER_FLOW = 32
    VOUCHER_VOUCHERTEXT1 = 33
    VOUCHER_PRODUCT_TEXT = 34
    VOUCHER_PRODUCT_SELECTED = 35
    VOUCHER_NODE_ID = 36
    VOUCHER_CHECKSUM1 = 37
    VOUCHER_ERROR_NUMBERS = 38
    VOUCHER_VOLUME_CORRECTION_FACTOR = 39
    VOUCHER_DENSITY_T = 40
    VOUCHER_DENSITY_X = 41
    VOUCHER_USER_FLOAT_1 = 42
    VOUCHER_USER_FLOAT_2 = 43
    VOUCHER_USER_FLOAT_3 = 44
    VOUCHER_USER_FLOAT_4 = 45
    VOUCHER_USER_FLOAT_5 = 46
    VOUCHER_USER_FLOAT_6 = 47
    VOUCHER_USER_UINT_1 = 48
    VOUCHER_USER_UINT_2 = 49
    VOUCHER_USER_UINT_3 = 50
    VOUCHER_USER_UINT_4 = 51
    VOUCHER_HOSTTEXT2 = 52
    VOUCHER_HOSTTEXT3 = 53
    VOUCHER_CHECKSUM2 = 54
    VOUCHER_TIMESTART = 55
    VOUCHER_RAW_PULSES = 56
    VOUCHER_PRESSURE = 57
    VOUCHER_AVERAGE_PRESSURE = 58
    VOUCHER_VT_PRESSURE_P0 = 59
    VOUCHER_VX_PRESSURE_P0 = 60
    VOUCHER_SUM_VT_PRESSURE_P0 = 61
    VOUCHER_SUM_VX_PRESSURE_P0 = 62
    VOUCHER_DENSITY_T_AT_P0_AVERAGE = 63
    VOUCHER_DENSITY_X_AT_P0_AVERAGE = 64
    VOUCHER_VOLUME_CORRECTION_FACTOR_TO_P0 = 65
    VOUCHER_VOLUME_CORRECTION_FACTOR_TO_P0_X = 66
    VOUCHER_VOLUME_CORRECTION_FACTOR_TO_X = 67
    VOUCHER_K_FACTOR = 68
    VOUCHER_METER_FACTOR = 69
    VOUCHER_CHECKSUM3 = 70
    VOUCHER_NUMBER_OF_ENTRIES = 71
    VOUCHER_HISTORY_AMOUNT_OF_STORED = 72
    VOUCHER_HISTORY_AMOUNT_OF_NOT_PRINTED = 73
    VOUCHER_HISTORY_AMOUNT_OF_NOT_ACKNOWLEDGED = 74
    VOUCHER_HISTORY_STATE = 75
    VOUCHER_HISTORY_COMMAND = 76
    VOUCHER_HISTORY_AMOUNT_MAX = 77
    VOUCHER_VALUE_COUNT = 78


class CWatchdogEnum(IntEnum):
    WATCHDOG_RESTARTCODE = 0
    WATCHDOG_VALUE_COUNT = 1


class CWMHistoryEnum(IntEnum):
    WMHIST_FIRST = 0
    WMHIST_LOG_1 = 1
    WMHIST_LOG_2 = 2
    WMHIST_LOG_3 = 3
    WMHIST_LOG_4 = 4
    WMHIST_LOG_5 = 5
    WMHIST_LOG_6 = 6
    WMHIST_LOG_7 = 7
    WMHIST_LOG_8 = 8
    WMHIST_LOG_9 = 9
    WMHIST_LOG_10 = 10
    WMHIST_LOG_11 = 11
    WMHIST_LOG_12 = 12
    WMHIST_LOG_13 = 13
    WMHIST_LOG_14 = 14
    WMHIST_LOG_15 = 15
    WMHIST_LOG_16 = 16
    WMHIST_LOG_17 = 17
    WMHIST_LOG_18 = 18
    WMHIST_LOG_19 = 19
    WMHIST_LOG_20 = 20
    WMHIST_LOG_21 = 21
    WMHIST_LOG_22 = 22
    WMHIST_LOG_23 = 23
    WMHIST_LOG_24 = 24
    WMHIST_LOG_25 = 25
    WMHIST_LOG_26 = 26
    WMHIST_LOG_27 = 27
    WMHIST_LOG_28 = 28
    WMHIST_LOG_29 = 29
    WMHIST_LOG_30 = 30
    WMHIST_LOG_31 = 31
    WMHIST_LOG_32 = 32
    WMHIST_LOG_33 = 33
    WMHIST_LOG_34 = 34
    WMHIST_LOG_35 = 35
    WMHIST_LOG_36 = 36
    WMHIST_LOG_37 = 37
    WMHIST_LOG_38 = 38
    WMHIST_LOG_39 = 39
    WMHIST_LOG_40 = 40
    WMHIST_LOG_41 = 41
    WMHIST_LOG_42 = 42
    WMHIST_LOG_43 = 43
    WMHIST_LOG_44 = 44
    WMHIST_LOG_45 = 45
    WMHIST_LOG_46 = 46
    WMHIST_LOG_47 = 47
    WMHIST_LOG_48 = 48
    WMHIST_LOG_49 = 49
    WMHIST_LOG_50 = 50
    WMHIST_VALUE_COUNT = 51

class CModuleEnum(IntEnum):
    TYPE_LIST_START = 0
    TYPE_LOG = TYPE_LIST_START
    TYPE_INTERNAL_ADC = 1
    TYPE_SYSTEM = 2
    TYPE_CLOCK = 3
    TYPE_ACCESSCONTROL = 4
    TYPE_WATCHDOG = 5
    TYPE_ADCONVERTER = 6
    TYPE_DACONVERTER = 7
    TYPE_PIDCONTROL = 8
    TYPE_TEMPERATURE = 9
    TYPE_COUNTER = 10
    TYPE_BOARDDISPLAY = 11
    TYPE_BOARDRELAY= 12
    TYPE_BOARDDIGITALINPUT= 13
    TYPE_BOARDDIGITALOUTPUT= 14
    TYPE_BOARDIMPULSE= 15
    TYPE_PULSEGENERATOR= 16
    TYPE_SERIAL= 17
    TYPE_PROCESSCONTROL= 18
    TYPE_PROTOCOL_MFX4= 19
    TYPE_MEMORY_INTERFACE= 20
    TYPE_DENSITY= 21
    TYPE_PRESSURE= 22
    TYPE_ERROR = 23
    TYPE_PULSE = 24
    TYPE_WEIGHT_AND_MEASUREMENT_HISTORY = 25
    TYPE_PRODUCT = 26
    TYPE_VOUCHER = 27
    TYPE_LIST_END = 28
    TYPE_COUNT = TYPE_LIST_END
    TYPE_MAX = 0xFFFF

